@@ -33,6 +33,7 @@ module Data.Serialize (
     , encode, encodeLazy
     , decode, decodeLazy
 
+    , expect
     , module Data.Serialize.Get
     , module Data.Serialize.Put
     , module Data.Serialize.IEEE754
@@ -112,6 +113,16 @@ decode = runGet get
 decodeLazy :: Serialize a => L.ByteString -> Either String a
 decodeLazy  = runGetLazy get
 
+
+------------------------------------------------------------------------
+-- Combinators
+
+-- | Perform an action, failing if the read result does not match the argument
+--   provided.
+expect :: (Eq a, Serialize a) => a -> Get a
+expect x = get >>= \y -> if x == y then return x else mzero
+
+
 ------------------------------------------------------------------------
 -- Simple instances
 
@@ -244,6 +255,14 @@ instance (Serialize a,Integral a) => Serialize (R.Ratio a) where
 
 ------------------------------------------------------------------------
 
+-- Safely wrap `chr` to avoid exceptions. 
+-- `chr` source: http://hackage.haskell.org/package/base-4.7.0.2/docs/src/GHC-Char.html#chr
+chrEither :: Int -> Either String Char
+chrEither i
+  | i <= 0x10FFFF = Right (chr i) -- Or: C# (chr# i#)
+  | otherwise =
+     Left ("bad argument: " ++ show i)
+
 -- Char is serialised as UTF-8
 instance Serialize Char where
     put a | c <= 0x7f     = put (fromIntegral c :: Word8)
@@ -285,7 +304,11 @@ instance Serialize Char where
                                 z <- liftM (xor 0x80) getByte
                                 return (z .|. shiftL6 (y .|. shiftL6
                                         (x .|. shiftL6 (xor 0xf0 w))))
-        return $! chr r
+        case chrEither r of
+            Right r' -> 
+                return $! r'
+            Left err ->
+                fail err
 
 ------------------------------------------------------------------------
 -- Instances for the first few tuples
